#include <exception>
#include <string>
#include <sstream>
#include "Exception.hpp"


Kronos::Exception::Exception::Exception() : m_cause(nullptr) {
	initTrace();
}

Kronos::Exception::Exception::Exception(const std::string& message) : m_cause(nullptr), m_message(message) {
	initTrace();
}

Kronos::Exception::Exception::Exception(const std::string& message, const std::exception& cause) : m_cause(&cause), m_message(message) {
	initTrace();
}

Kronos::Exception::Exception::Exception(const std::exception& cause) : m_cause(&cause) {
	initTrace();
}

void Kronos::Exception::Exception::initTrace() {
	std::stringstream ss;

	const std::string& msg = getMessage();

	ss << typeid(*this).name();
	if(!msg.empty()) {
		ss << " : " << msg;
	}
	if(m_cause != nullptr) {
		ss << "\n    -> " << m_cause->what();
	}

	m_trace = ss.str();
}

const char* Kronos::Exception::Exception::what() const noexcept {
	return m_trace.c_str();
}

const std::exception* Kronos::Exception::Exception::getCause() const {
	return m_cause;
}

const std::string& Kronos::Exception::Exception::getMessage() const {
	return m_message;
}
